@@ -1,15 +1,17 @@
-__version__ = "1.1.1"
+__version__ = "2.0.0"
 
 # Imports
 import asyncio
-from abc import abstractmethod
-from datetime import datetime
 import enum
 import json
-import requests
-import sys
-from urllib.parse import quote
 import uuid
+from abc import abstractmethod
+from datetime import datetime
+from urllib.parse import quote
+
+import aiohttp
+import requests
+from aiohttp import ClientResponseError
 
 
 # Enums
@@ -42,10 +44,17 @@ class friendActions(enum.Enum):
 
 
 class API:
-    def __init__(self):
-        # common class
-        self.__common = self.__common()
+    """
+    Call Of Duty API Wrapper
 
+    Developed by Todo Lodo & Engineer152
+
+    Contributors
+    - Werseter
+
+    Source Code: https://github.com/TodoLodo/cod-python-api
+    """
+    def __init__(self):
         # sub classes
         self.Warzone = self.__WZ()
         self.ModernWarfare = self.__MW()
@@ -57,179 +66,101 @@ class API:
         self.Me = self.__USER()
         self.Misc = self.__ALT()
 
+    async def loginAsync(self, sso_token: str) -> None:
+        await API._Common.loginAsync(sso_token)
+
     # Login
     def login(self, ssoToken: str):
-        try:
-            self.__common.baseHeaders["__X-XSRF-TOKEN"] = self.__common.fakeXSRF
-            self.__common.baseHeaders["__X-CSRF-TOKEN"] = self.__common.fakeXSRF
-            self.__common.baseHeaders["Atvi-Auth"] = ssoToken
-            self.__common.baseHeaders["ACT_SSO_COOKIE"] = ssoToken
-            self.__common.baseHeaders["atkn"] = ssoToken
-            self.__common.baseHeaders["cookie"] = f'{self.__common.baseCookie}' \
-                                                  f'ACT_SSO_COOKIE={ssoToken};' \
-                                                  f'XSRF-TOKEN={self.__common.fakeXSRF};' \
-                                                  f'API_CSRF_TOKEN={self.__common.fakeXSRF};' \
-                                                  f'ACT_SSO_EVENT="LOGIN_SUCCESS:1644346543228";' \
-                                                  f'ACT_SSO_COOKIE_EXPIRY=1645556143194;' \
-                                                  f'comid=cod;' \
-                                                  f'ssoDevId=63025d09c69f47dfa2b8d5520b5b73e4;' \
-                                                  f'tfa_enrollment_seen=true;' \
-                                                  f'gtm.custom.bot.flag=human;'
-            self.__common.baseSsoToken = ssoToken
-            self.__common.basePostHeaders["__X-XSRF-TOKEN"] = self.__common.fakeXSRF
-            self.__common.basePostHeaders["__X-CSRF-TOKEN"] = self.__common.fakeXSRF
-            self.__common.basePostHeaders["Atvi-Auth"] = ssoToken
-            self.__common.basePostHeaders["ACT_SSO_COOKIE"] = ssoToken
-            self.__common.basePostHeaders["atkn"] = ssoToken
-            self.__common.basePostHeaders["cookie"] = f'{self.__common.baseCookie}' \
-                                                      f'ACT_SSO_COOKIE={ssoToken};' \
-                                                      f'XSRF-TOKEN={self.__common.fakeXSRF};' \
-                                                      f'API_CSRF_TOKEN={self.__common.fakeXSRF};' \
-                                                      f'ACT_SSO_EVENT="LOGIN_SUCCESS:1644346543228";' \
-                                                      f'ACT_SSO_COOKIE_EXPIRY=1645556143194;' \
-                                                      f'comid=cod;' \
-                                                      f'ssoDevId=63025d09c69f47dfa2b8d5520b5b73e4;' \
-                                                      f'tfa_enrollment_seen=true;' \
-                                                      f'gtm.custom.bot.flag=human;'
+        API._Common.login(ssoToken)
 
-            r = requests.get(f"{self.__common.baseUrl}{self.__common.apiPath}/crm/cod/v2/identities/{ssoToken}",
-                             headers=self.__common.baseHeaders)
+    class _Common:
+        requestHeaders = {
+            "content-type": "application/json",
+            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
+                          "AppleWebKit/537.36 (KHTML, like Gecko) "
+                          "Chrome/74.0.3729.169 "
+                          "Safari/537.36",
+            "Accept": "application/json",
+            "Connection": "Keep-Alive"
+        }
+        cookies = {"new_SiteId": "cod", "ACT_SSO_LOCALE": "en_US", "country": "US",
+                   "ACT_SSO_COOKIE_EXPIRY": "1645556143194"}
+        cachedMappings = None
+
+        fakeXSRF = str(uuid.uuid4())
+        baseUrl: str = "https://my.callofduty.com/api/papi-client"
+        loggedIn: bool = False
+
+        # endPoints
+
+        # game platform lookupType gamertag type
+        fullDataUrl = "/stats/cod/v1/title/%s/platform/%s/%s/%s/profile/type/%s"
+        # game platform lookupType gamertag type start end [?limit=n or '']
+        combatHistoryUrl = "/crm/cod/v2/title/%s/platform/%s/%s/%s/matches/%s/start/%d/end/%d/details"
+        # game platform lookupType gamertag type start end
+        breakdownUrl = "/crm/cod/v2/title/%s/platform/%s/%s/%s/matches/%s/start/%d/end/%d"
+        # game platform lookupType gamertag
+        seasonLootUrl = "/loot/title/%s/platform/%s/%s/%s/status/en"
+        # game platform
+        mapListUrl = "/ce/v1/title/%s/platform/%s/gameType/mp/communityMapData/availability"
+        # game platform type matchId
+        matchInfoUrl = "/crm/cod/v2/title/%s/platform/%s/fullMatch/%s/%d/en"
+
+        @staticmethod
+        async def loginAsync(sso_token: str) -> None:
+            API._Common.cookies["ACT_SSO_COOKIE"] = sso_token
+            API._Common.baseSsoToken = sso_token
+            r = await API._Common.__Request(f"{API._Common.baseUrl}/crm/cod/v2/identities/{sso_token}")
+            if r['status'] == 'success':
+                API._Common.loggedIn = True
+            else:
+                raise InvalidToken(sso_token)
+
+        @staticmethod
+        def login(sso_token: str) -> None:
+            API._Common.cookies["ACT_SSO_COOKIE"] = sso_token
+            API._Common.baseSsoToken = sso_token
+
+            r = requests.get(f"{API._Common.baseUrl}/crm/cod/v2/identities/{sso_token}",
+                             headers=API._Common.requestHeaders, cookies=API._Common.cookies)
 
             if r.json()['status'] == 'success':
-                self.__common.loggedIn = True
-                for sub in [self.Warzone, self.Warzone2, self.ModernWarfare, self.ModernWarfare2, self.ColdWar,
-                            self.Vanguard, self.Shop, self.Me, self.Misc]:
-                    sub.loggedIn = self.__common.loggedIn
-                    sub.baseSsoToken = self.__common.baseSsoToken
-                    sub.baseHeaders = self.__common.baseHeaders
-                    sub.basePostHeaders = self.__common.basePostHeaders
-
-                # deleting scope data
-                del ssoToken, sub, r
+                API._Common.loggedIn = True
+                API._Common.cookies.update(r.cookies)
             else:
-                # delete scope data
-                del ssoToken, r
+                raise InvalidToken(sso_token)
 
-                # system exit
-                sys.exit(InvalidToken(ssoToken))
-        except Exception as e:
-            print(e)
-
-            # delete scope data
-            del ssoToken
-
-            return e
-
-    class __common:
-        customHeaders: dict = {
-            "__X-XSRF-TOKEN": str or None,
-            "__X-CSRF-TOKEN": str or None,
-            "Atvi-Auth": str or None,
-            "ACT_SSO_COOKIE": str or None,
-            "atkn": str or None,
-            "cookie": str or None,
-            "content-type": str or None,
-        }
-        customBody: dict = {}
-
-        def __init__(self):
-            # Variables
-
-            # headers & cookies
-            self.fakeXSRF = str(uuid.uuid4())
-            self.userAgent: str = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) " \
-                                  "AppleWebKit/537.36 (KHTML, like Gecko) " \
-                                  "Chrome/74.0.3729.169 " \
-                                  "Safari/537.36"
-            self.baseCookie: str = "new_SiteId=cod;ACT_SSO_LOCALE=en_US;country=US;"
-            self.baseSsoToken: str = ''
-            self.baseUrl: str = "https://my.callofduty.com"
-            self.apiPath: str = "/api/papi-client"
-            self.loggedIn: bool = False
-
-            # headers structures
-            self.baseHeaders: API.customHeaders = {
-                "content-type": 'application/json',
-                "cookie": self.baseCookie,
-                "user-agent": self.userAgent
-            }
-
-            self.basePostHeaders: API.customHeaders = {
-                "content-type": 'text/plain',
-                "cookie": self.baseCookie,
-                "user-agent": self.userAgent
-            }
-
-            # endPoints
-
-            # game platform lookupType gamertag type
-            self.fullDataUrl = "/stats/cod/v1/title/%s/platform/%s/%s/%s/profile/type/%s"
-            # game platform lookupType gamertag type start end [?limit=n or '']
-            self.combatHistoryUrl = "/crm/cod/v2/title/%s/platform/%s/%s/%s/matches/%s/start/%d/end/%d/details"
-            # game platform lookupType gamertag type start end
-            self.breakdownUrl = "/crm/cod/v2/title/%s/platform/%s/%s/%s/matches/%s/start/%d/end/%d"
-            # game platform lookupType gamertag
-            self.seasonLootUrl = "/loot/title/%s/platform/%s/%s/%s/status/en"
-            # game platform
-            self.mapListUrl = "/ce/v1/title/%s/platform/%s/gameType/mp/communityMapData/availability"
-            # game platform type matchId
-            self.matchInfoUrl = "/crm/cod/v2/title/%s/platform/%s/fullMatch/%s/%d/en"
+        @staticmethod
+        def sso_token() -> str:
+            return API._Common.cookies["ACT_SSO_COOKIE"]
 
         # Requests
-        async def __Request(self, method, url):
-            h: self.customHeaders = self.customHeaders
-            b: self.customBody = self.customBody
-            if method == "GET":
-                h = self.baseHeaders
-            elif method == "POST":
-                h = self.basePostHeaders
 
-            try:
-                r = requests.request(method=method, url=url, headers=h, data=b)
-
-                # return data
-                return r
-            except Exception as e:
-
-                return e
+        @staticmethod
+        async def __Request(url):
+            async with aiohttp.client.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=True),
+                                                    timeout=aiohttp.ClientTimeout(total=30)) as session:
+                try:
+                    async with session.get(url, cookies=API._Common.cookies,
+                                           headers=API._Common.requestHeaders) as resp:
+                        try:
+                            resp.raise_for_status()
+                        except ClientResponseError as err:
+                            return {'status': 'error', 'data': {'type': type(err), 'message': err.message}}
+                        else:
+                            API._Common.cookies.update({c.key: c.value for c in session.cookie_jar})
+                            return await resp.json()
+                except asyncio.TimeoutError as err:
+                    return {'status': 'error', 'data': {'type': type(err), 'message': str(err)}}
 
         async def __sendRequest(self, url: str):
             if self.loggedIn:
-                respond = await self.__Request("GET", f"{self.baseUrl}{self.apiPath}{url}")
-                if type(respond) != Exception:
-                    if respond.status_code == 200:
-                        data = respond.json()
-                        if data['status'] == 'success':
-                            data = self.__mapping(data['data'])
-                            # delete scope data
-                            del url, respond
-
-                            # return data
-                            return data
-                        else:
-                            # delete scope data
-                            del url, respond
-
-                            sys.exit(StatusError())
-                    else:
-                        # delete scope data
-                        del url
-
-                        return respond.status_code
+                response = await API._Common.__Request(f"{self.baseUrl}{url}")
+                if response['status'] == 'success':
+                    response['data'] = await self.__perform_mapping(response['data'])
+                return response
             else:
-                sys.exit(NotLoggedIn())
-
-        async def __sendPostRequest(self, url: str, body: dict):
-            if self.loggedIn:
-                customBody = body
-                respond = await self.__Request("POST", f"{self.baseUrl}{self.apiPath}{url}")
-                if type(respond) != Exception:
-                    if respond.status_code == 200:
-                        return respond.json()
-                    else:
-                        return respond.status_code
-            else:
-                raise NotLoggedIn()
+                raise NotLoggedIn
 
         # client name url formatter
         def __cleanClientName(self, gamertag):
@@ -246,23 +177,22 @@ class API:
                                 platforms.XBOX]:
                 raise InvalidPlatform(platform)
             else:
-                if platform in [platforms.Activision, platforms.Battlenet, platforms.Uno]:
-                    gamertag = self.__cleanClientName(gamertag)
+                gamertag = self.__cleanClientName(gamertag)
             return lookUpType, gamertag, platform
 
-        # mapping
-        def __mapping(self, data):
-            r = requests.get(
-                'https://engineer152.github.io/wz-data/weapon-ids.json')
-            guns = r.json()
-            r = requests.get(
-                'https://engineer152.github.io/wz-data/game-modes.json')
-            modes = r.json()
-            r = requests.get(
-                'https://engineer152.github.io/wz-data/perks.json')
-            perks = r.json()
+        async def __get_mappings(self):
+            if API._Common.cachedMappings is None:
+                API._Common.cachedMappings = (
+                    await API._Common.__Request('https://engineer152.github.io/wz-data/weapon-ids.json'),
+                    await API._Common.__Request('https://engineer152.github.io/wz-data/game-modes.json'),
+                    await API._Common.__Request('https://engineer152.github.io/wz-data/perks.json'))
+            return API._Common.cachedMappings
 
-            # guns
+        # mapping
+        async def __perform_mapping(self, data):
+            guns, modes, perks = await self.__get_mappings()
+            if not isinstance(data, list) or 'matches' not in data:
+                return data
             try:
                 for match in data['matches']:
                     # time stamps
@@ -334,14 +264,6 @@ class API:
                                         pass
             except KeyError:
                 pass
-            except Exception as e:
-                print(e)
-
-            # delete scope data
-            try:
-                del guns, modes, perks, match, loadout, perk
-            except UnboundLocalError:
-                pass
 
             # return mapped or unmapped data
             return data
@@ -371,7 +293,66 @@ class API:
         async def _matchInfoReq(self, game, platform, type, matchId):
             return await self.__sendRequest(self.matchInfoUrl % (game, platform.value, type, matchId))
 
-    class __GameDataCommons(__common):
+    class __GameDataCommons(_Common):
+        """
+        Methods
+        =======
+        Sync
+        ----
+        fullData(platform:platforms, gamertag:str)
+            returns player's game data of type dict
+
+        combatHistory(platform:platforms, gamertag:str)
+            returns player's combat history of type dict
+
+        combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
+            returns player's combat history within the specified timeline of type dict
+
+        breakdown(platform:platforms, gamertag:str)
+            returns player's combat history breakdown of type dict
+
+        breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
+                    returns player's combat history breakdown within the specified timeline of type dict
+
+        seasonLoot(platform:platforms, gamertag:str)
+            returns player's season loot
+
+        mapList(platform:platforms)
+            returns available maps and available modes for each
+
+        matchInfo(platform:platforms, matchId:int)
+                    returns details match details of type dict
+
+        Async
+        ----
+        fullDataAsync(platform:platforms, gamertag:str)
+            returns player's game data of type dict
+
+        combatHistoryAsync(platform:platforms, gamertag:str)
+            returns player's combat history of type dict
+
+        combatHistoryWithDateAsync(platform:platforms, gamertag:str, start:int, end:int)
+            returns player's combat history within the specified timeline of type dict
+
+        breakdownAsync(platform:platforms, gamertag:str)
+            returns player's combat history breakdown of type dict
+
+        breakdownWithDateAsync(platform:platforms, gamertag:str, start:int, end:int)
+                    returns player's combat history breakdown within the specified timeline of type dict
+
+        seasonLootAsync(platform:platforms, gamertag:str)
+            returns player's season loot
+
+        mapListAsync(platform:platforms)
+            returns available maps and available modes for each
+
+        matchInfoAsync(platform:platforms, matchId:int)
+                    returns details match details of type dict
+        """
+
+        def __init_subclass__(cls, **kwargs):
+            cls.__doc__ = cls.__doc__ + super(cls, cls).__doc__
+
         @property
         @abstractmethod
         def _game(self) -> str:
@@ -442,29 +423,10 @@ class API:
     class __WZ(__GameDataCommons):
         """
         Warzone class: A class to get players warzone stats, warzone combat history and specific warzone match details
-            classCatogery: game
+            classCategory: game
             gameId/gameTitle: mw or wz
             gameType: wz
 
-        Methods
-        -------
-        fullData(platform:platforms, gamertagLstr)
-            returns player's game data of type dict
-
-        combatHistory(platform:platforms, gamertag:str)
-            returns player's combat history of type dict
-
-        combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
-            returns player's combat history within the specified timeline of type dict
-
-        breakdown(platform:platforms, gamertag:str)
-            returns player's combat history breakdown of type dict
-
-        breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
-                    returns player's combat history breakdown within the specified timeline of type dict
-
-        matchInfo(platform:platforms, matchId:int)
-                    returns details match details of type dict
         """
 
         @property
@@ -486,29 +448,10 @@ class API:
     class __WZ2(__GameDataCommons):
         """
         Warzone 2 class: A class to get players warzone 2 stats, warzone 2 combat history and specific warzone 2 match details
-            classCatogery: game
+            classCategory: game
             gameId/gameTitle: mw or wz
             gameType: wz2
 
-        Methods
-        -------
-        fullData(platform:platforms, gamertagLstr)
-            returns player's game data of type dict
-
-        combatHistory(platform:platforms, gamertag:str)
-            returns player's combat history of type dict
-
-        combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
-            returns player's combat history within the specified timeline of type dict
-
-        breakdown(platform:platforms, gamertag:str)
-            returns player's combat history breakdown of type dict
-
-        breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
-                    returns player's combat history breakdown within the specified timeline of type dict
-
-        matchInfo(platform:platforms, matchId:int)
-                    returns details match details of type dict
         """
 
         @property
@@ -530,35 +473,10 @@ class API:
     class __MW(__GameDataCommons):
         """
         ModernWarfare class: A class to get players modernwarfare stats, modernwarfare combat history, a player's modernwarfare season loot, modernwarfare map list and specific modernwarfare match details
-            classCatogery: game
+            classCategory: game
             gameId/gameTitle: mw
             gameType: mp
 
-        Methods
-        -------
-        fullData(platform:platforms, gamertagLstr)
-            returns player's game data of type dict
-
-        combatHistory(platform:platforms, gamertag:str)
-            returns player's combat history of type dict
-
-        combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
-            returns player's combat history within the specified timeline of type dict
-
-        breakdown(platform:platforms, gamertag:str)
-            returns player's combat history breakdown of type dict
-
-        breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
-                    returns player's combat history breakdown within the specified timeline of type dict
-
-        seasonLoot(platform:platforms, gamertag:str)
-            returns player's season loot
-
-        mapList(platform:platforms)
-            returns available maps and available modes for each
-
-        matchInfo(platform:platforms, matchId:int)
-                    returns details match details of type dict
         """
 
         @property
@@ -574,37 +492,11 @@ class API:
     class __CW(__GameDataCommons):
         """
          ColdWar class: A class to get players coldwar stats, coldwar combat history, a player's coldwar season loot, coldwar map list and specific coldwar match details
-             classCatogery: game
+             classCategory: game
              gameId/gameTitle: cw
              gameType: mp
 
-         Methods
-         -------
-         fullData(platform:platforms, gamertagLstr)
-             returns player's game data of type dict
-
-         combatHistory(platform:platforms, gamertag:str)
-             returns player's combat history of type dict
-
-         combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
-             returns player's combat history within the specified timeline of type dict
-
-         breakdown(platform:platforms, gamertag:str)
-             returns player's combat history breakdown of type dict
-
-         breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
-                     returns player's combat history breakdown within the specified timeline of type dict
-
-         seasonLoot(platform:platforms, gamertag:str)
-             returns player's season loot
-
-         mapList(platform:platforms)
-             returns available maps and available modes for each
-
-         matchInfo(platform:platforms, matchId:int)
-                     returns details match details of type dict
          """
-
         @property
         def _game(self) -> str:
             return "cw"
@@ -618,35 +510,10 @@ class API:
     class __VG(__GameDataCommons):
         """
          Vanguard class: A class to get players vanguard stats, vanguard combat history, a player's vanguard season loot, vanguard map list and specific vanguard match details
-             classCatogery: game
+             classCategory: game
              gameId/gameTitle: vg
              gameType: pm
 
-         Methods
-         -------
-         fullData(platform:platforms, gamertagLstr)
-             returns player's game data of type dict
-
-         combatHistory(platform:platforms, gamertag:str)
-             returns player's combat history of type dict
-
-         combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
-             returns player's combat history within the specified timeline of type dict
-
-         breakdown(platform:platforms, gamertag:str)
-             returns player's combat history breakdown of type dict
-
-         breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
-                     returns player's combat history breakdown within the specified timeline of type dict
-
-         seasonLoot(platform:platforms, gamertag:str)
-             returns player's season loot
-
-         mapList(platform:platforms)
-             returns available maps and available modes for each
-
-         matchInfo(platform:platforms, matchId:int)
-                     returns details match details of type dict
          """
 
         @property
@@ -662,35 +529,10 @@ class API:
     class __MW2(__GameDataCommons):
         """
         ModernWarfare 2 class: A class to get players modernwarfare 2 stats, modernwarfare 2 combat history, a player's modernwarfare 2 season loot, modernwarfare 2 map list and specific modernwarfare 2 match details
-            classCatogery: game
+            classCategory: game
             gameId/gameTitle: mw
             gameType: mp
 
-        Methods
-        -------
-        fullData(platform:platforms, gamertagLstr)
-            returns player's game data of type dict
-
-        combatHistory(platform:platforms, gamertag:str)
-            returns player's combat history of type dict
-
-        combatHistoryWithDate(platform:platforms, gamertag:str, start:int, end:int)
-            returns player's combat history within the specified timeline of type dict
-
-        breakdown(platform:platforms, gamertag:str)
-            returns player's combat history breakdown of type dict
-
-        breakdownWithDate(platform:platforms, gamertag:str, start:int, end:int)
-                    returns player's combat history breakdown within the specified timeline of type dict
-
-        seasonLoot(platform:platforms, gamertag:str)
-            returns player's season loot
-
-        mapList(platform:platforms)
-            returns available maps and available modes for each
-
-        matchInfo(platform:platforms, matchId:int)
-                    returns details match details of type dict
         """
 
         @property
@@ -702,20 +544,15 @@ class API:
             return "mp"
 
     # USER
-    class __USER(__common):
+    class __USER(_Common):
         def info(self):
             if self.loggedIn:
-                headers = self.baseHeaders
-                headers['Accept'] = 'application/json'
-                rawData = requests.get(
-                    f"https://profile.callofduty.com/cod/userInfo/{self.baseSsoToken}", headers=headers)
+                rawData = requests.get(f"https://profile.callofduty.com/cod/userInfo/{self.sso_token()}",
+                                       headers=API._Common.requestHeaders)
                 rawData = json.loads(rawData.text.replace(
                     'userInfo(', '').replace(');', ''))
 
-                data = {
-                    'userName': rawData['userInfo']['userName']
-                }
-                data['identities'] = []
+                data = {'userName': rawData['userInfo']['userName'], 'identities': []}
                 for i in rawData['identities']:
                     data['identities'].append({
                         'platform': i['provider'],
@@ -724,7 +561,7 @@ class API:
                     })
                 return data
             else:
-                sys.exit(NotLoggedIn())
+                raise NotLoggedIn
 
         def __priv(self):
             d = self.info()
@@ -732,7 +569,7 @@ class API:
 
         async def friendFeedAsync(self):
             p, g = self.__priv()
-            data = await self._common__sendRequest(
+            data = await self._Common__sendRequest(
                     f"/userfeed/v1/friendFeed/platform/{p}/gamer/{g}/friendFeedEvents/en")
             return data
 
@@ -740,14 +577,14 @@ class API:
             return asyncio.run(self.friendFeedAsync())
 
         async def eventFeedAsync(self):
-            data = await self._common__sendRequest(f"/userfeed/v1/friendFeed/rendered/en/{self.baseSsoToken}")
+            data = await self._Common__sendRequest(f"/userfeed/v1/friendFeed/rendered/en/{self.sso_token()}")
             return data
 
         def eventFeed(self):
             return asyncio.run(self.eventFeedAsync())
 
         async def loggedInIdentitiesAsync(self):
-            data = await self._common__sendRequest(f"/crm/cod/v2/identities/{self.baseSsoToken}")
+            data = await self._Common__sendRequest(f"/crm/cod/v2/identities/{self.sso_token()}")
             return data
 
         def loggedInIdentities(self):
@@ -755,7 +592,7 @@ class API:
 
         async def codPointsAsync(self):
             p, g = self.__priv()
-            data = await self._common__sendRequest(f"/inventory/v1/title/mw/platform/{p}/gamer/{g}/currency")
+            data = await self._Common__sendRequest(f"/inventory/v1/title/mw/platform/{p}/gamer/{g}/currency")
             return data
 
         def codPoints(self):
@@ -763,7 +600,7 @@ class API:
 
         async def connectedAccountsAsync(self):
             p, g = self.__priv()
-            data = await self._common__sendRequest(f"/crm/cod/v2/accounts/platform/{p}/gamer/{g}")
+            data = await self._Common__sendRequest(f"/crm/cod/v2/accounts/platform/{p}/gamer/{g}")
             return data
 
         def connectedAccounts(self):
@@ -771,20 +608,22 @@ class API:
 
         async def settingsAsync(self):
             p, g = self.__priv()
-            data = await self._common__sendRequest(f"/preferences/v1/platform/{p}/gamer/{g}/list")
+            data = await self._Common__sendRequest(f"/preferences/v1/platform/{p}/gamer/{g}/list")
             return data
 
         def settings(self):
             return asyncio.run(self.settingsAsync())
 
     # SHOP
-    class __SHOP(__common):
+    class __SHOP(_Common):
         """
          Shop class: A class to get bundle details and battle pass loot
-             classCatogery: other
+             classCategory: other
 
          Methods
-         -------
+         =======
+         Sync
+         ----
          purchasableItems(game: games)
              returns purchasable items for a specific gameId/gameTitle
 
@@ -793,24 +632,35 @@ class API:
 
          battlePassLoot(game: games, platform: platforms, season: int)
              returns battle pass loot for specific game and season on given platform
+
+        Async
+         ----
+         purchasableItemsAsync(game: games)
+             returns purchasable items for a specific gameId/gameTitle
+
+         bundleInformationAsync(game: games, bundleId: int)
+             returns bundle details for the specific gameId/gameTitle and bundleId
+
+         battlePassLootAsync(game: games, platform: platforms, season: int)
+             returns battle pass loot for specific game and season on given platform
          """
 
         async def purchasableItemsAsync(self, game: games):
-            data = await self._common__sendRequest(f"/inventory/v1/title/{game}/platform/uno/purchasable/public/en")
+            data = await self._Common__sendRequest(f"/inventory/v1/title/{game}/platform/uno/purchasable/public/en")
             return data
 
         def purchasableItems(self, game: games):
             return asyncio.run(self.purchasableItemsAsync(game))
 
         async def bundleInformationAsync(self, game: games, bundleId: int):
-            data = await self._common__sendRequest(f"/inventory/v1/title/{game}/bundle/{bundleId}/en")
+            data = await self._Common__sendRequest(f"/inventory/v1/title/{game}/bundle/{bundleId}/en")
             return data
 
         def bundleInformation(self, game: games, bundleId: int):
             return asyncio.run(self.bundleInformationAsync(game, bundleId))
 
         async def battlePassLootAsync(self, game: games, platform: platforms, season: int):
-            data = await self._common__sendRequest(
+            data = await self._Common__sendRequest(
                 f"/loot/title/{game}/platform/{platform.value}/list/loot_season_{season}/en")
             return data
 
@@ -818,11 +668,11 @@ class API:
             return asyncio.run(self.battlePassLootAsync(game, platform, season))
 
     # ALT
-    class __ALT(__common):
+    class __ALT(_Common):
 
         async def searchAsync(self, platform, gamertag: str):
-            lookUpType, gamertag, platform = self._helper(platform, gamertag)
-            data = await self._common__sendRequest(f"/crm/cod/v2/platform/{platform.value}/username/{gamertag}/search")
+            lookUpType, gamertag, platform = self._Common__helper(platform, gamertag)
+            data = await self._Common__sendRequest(f"/crm/cod/v2/platform/{platform.value}/username/{gamertag}/search")
             return data
 
         def search(self, platform, gamertag: str):
@@ -852,6 +702,9 @@ class InvalidPlatform(Exception):
         else:
             self.message = "Invalid platform, use platform class!"
 
+
+        super().__init__(self.message)
+
     def __str__(self):
         return self.message
 
